@@ -1,26 +1,26 @@
-#include "common/AsyncTask.h"
 #include "InitialOptionsDialog.h"
+#include "common/AsyncTask.h"
 #include "ui_InitialOptionsDialog.h"
 
-#include "core/MainWindow.h"
-#include "dialogs/NewFileDialog.h"
-#include "dialogs/AsyncTaskDialog.h"
 #include "common/Helpers.h"
+#include "core/MainWindow.h"
+#include "dialogs/AsyncTaskDialog.h"
+#include "dialogs/NewFileDialog.h"
 
-#include <QSettings>
-#include <QFileInfo>
-#include <QFileDialog>
 #include <QCloseEvent>
+#include <QFileDialog>
+#include <QFileInfo>
+#include <QSettings>
 
-#include "core/Iaito.h"
 #include "common/AnalTask.h"
+#include "core/Iaito.h"
 
-
-InitialOptionsDialog::InitialOptionsDialog(MainWindow *main):
-    QDialog(nullptr), // parent must not be main
-    ui(new Ui::InitialOptionsDialog),
-    main(main),
-    core(Core())
+InitialOptionsDialog::InitialOptionsDialog(MainWindow *main)
+    : QDialog(nullptr)
+    , // parent must not be main
+    ui(new Ui::InitialOptionsDialog)
+    , main(main)
+    , core(Core())
 {
     ui->setupUi(this);
     setWindowFlags(windowFlags() & (~Qt::WindowContextHelpButtonHint));
@@ -46,7 +46,8 @@ InitialOptionsDialog::InitialOptionsDialog(MainWindow *main):
         }
     }
 
-    setTooltipWithConfigHelp(ui->archComboBox,"asm.arch");
+    setTooltipWithConfigHelp(ui->archComboBox, "asm.arch");
+    ui->archComboBox->model()->sort(0);
 
     // cpu combo box
     ui->cpuComboBox->lineEdit()->setPlaceholderText(tr("Auto"));
@@ -67,31 +68,34 @@ InitialOptionsDialog::InitialOptionsDialog(MainWindow *main):
     }
 
     analysisCommands = {
-        { { "aa", tr("Analyze all symbols") }, new QCheckBox(), true },
-        { { "aar", tr("Analyze instructions for references") }, new QCheckBox(), true },
-        { { "aac", tr("Analyze function calls") }, new QCheckBox(), true },
-        { { "aab", tr("Analyze all basic blocks") }, new QCheckBox(), false },
-        { { "aao", tr("Analyze all objc references") }, new QCheckBox(), false },
-        { { "avrr", tr("Recover class information from RTTI") }, new QCheckBox(), false },
-        { { "aan", tr("Autoname functions based on context") }, new QCheckBox(), false },
-        { { "aae", tr("Emulate code to find computed references") }, new QCheckBox(), false },
-        { { "aafr", tr("Analyze all consecutive functions") }, new QCheckBox(), false },
-        { { "aaft", tr("Type and Argument matching analysis") }, new QCheckBox(), false },
-        { { "aaT", tr("Analyze code after trap-sleds") }, new QCheckBox(), false },
-        { { "aap", tr("Analyze function preludes") }, new QCheckBox(), false },
-        { { "e! anal.jmp.tbl", tr("Analyze jump tables in switch statements") }, new QCheckBox(), false },
-        { { "e! anal.pushret", tr("Analyze PUSH+RET as JMP") },  new QCheckBox(), false },
-        { { "e! anal.hasnext", tr("Continue analysis after each function") }, new QCheckBox(), false }};
+        {{"aa", tr("Analyze all symbols")}, new QCheckBox(), true},
+        {{"aar", tr("Analyze instructions for references")}, new QCheckBox(), true},
+        {{"aac", tr("Analyze function calls")}, new QCheckBox(), true},
+        {{"aab", tr("Analyze all basic blocks")}, new QCheckBox(), false},
+        {{"aao", tr("Analyze all objc references")}, new QCheckBox(), false},
+        {{"avrr", tr("Recover class information from RTTI")}, new QCheckBox(), false},
+        {{"aan", tr("Autoname functions based on context")}, new QCheckBox(), false},
+        {{"aae", tr("Linearly emulate code to find computed references")}, new QCheckBox(), false},
+        {{"aaef", tr("Emulate all functions to find computed references")}, new QCheckBox(), false},
+        {{"aafr", tr("Analyze all consecutive functions")}, new QCheckBox(), false},
+        {{"aaft", tr("Type and Argument matching analysis")}, new QCheckBox(), false},
+        {{"aaT", tr("Analyze code after trap-sleds")}, new QCheckBox(), false},
+        {{"aap", tr("Analyze function preludes")}, new QCheckBox(), false},
+        {{"aaw", tr("Analyze data accesses as dwords")}, new QCheckBox(), false},
+        {{"e! anal.jmp.tbl", tr("Analyze jump tables in switch statements")},
+         new QCheckBox(),
+         false},
+        {{"e! anal.pushret", tr("Analyze PUSH+RET as JMP")}, new QCheckBox(), false},
+        {{"e! anal.hasnext", tr("Continue analysis after each function")}, new QCheckBox(), false}};
 
     // Per each checkbox, set a tooltip desccribing it
     AnalysisCommands item;
-    foreach (item, analysisCommands){
+    foreach (item, analysisCommands) {
         item.checkbox->setText(item.commandDesc.description);
         item.checkbox->setToolTip(item.commandDesc.command);
         item.checkbox->setChecked(item.checked);
         ui->verticalLayout_7->addWidget(item.checkbox);
     }
-    
 
     ui->hideFrame->setVisible(false);
     ui->analoptionsFrame->setVisible(false);
@@ -103,7 +107,11 @@ InitialOptionsDialog::InitialOptionsDialog(MainWindow *main):
 
     updateScriptLayout();
 
-    connect(ui->scriptCheckBox, &QCheckBox::stateChanged, this, &InitialOptionsDialog::updateScriptLayout);
+    connect(
+        ui->scriptCheckBox,
+        &QCheckBox::stateChanged,
+        this,
+        &InitialOptionsDialog::updateScriptLayout);
 
     connect(ui->cancelButton, &QPushButton::clicked, this, &InitialOptionsDialog::reject);
 
@@ -120,11 +128,12 @@ void InitialOptionsDialog::updateCPUComboBox()
     QString arch = getSelectedArch();
     QStringList cpus;
     if (!arch.isEmpty()) {
-        auto pluginDescr = std::find_if(asmPlugins.begin(), asmPlugins.end(), [&](const RAsmPluginDescription &plugin) {
-            return plugin.name == arch;
-        });
+        auto pluginDescr = std::find_if(
+            asmPlugins.begin(), asmPlugins.end(), [&](const RAsmPluginDescription &plugin) {
+                return plugin.name == arch;
+            });
         if (pluginDescr != asmPlugins.end()) {
-#if QT_VERSION >= QT_VERSION_CHECK(5,14,0)
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
             cpus = pluginDescr->cpus.split(",", Qt::SkipEmptyParts);
 #else
             cpus = pluginDescr->cpus.split(",", QString::SkipEmptyParts);
@@ -138,9 +147,10 @@ void InitialOptionsDialog::updateCPUComboBox()
     ui->cpuComboBox->lineEdit()->setText(currentText);
 }
 
-QList<QString> InitialOptionsDialog::getAnalysisCommands(const InitialOptions &options) {
+QList<QString> InitialOptionsDialog::getAnalysisCommands(const InitialOptions &options)
+{
     QList<QString> commands;
-    for (auto& commandDesc: options.analCmd) {
+    for (auto &commandDesc : options.analCmd) {
         commands << commandDesc.command;
     }
     return commands;
@@ -150,17 +160,21 @@ void InitialOptionsDialog::loadOptions(const InitialOptions &options)
 {
     if (options.analCmd.isEmpty()) {
         analLevel = 0;
-    } else if (options.analCmd.first().command == "aa" ) {
+    } else if (options.analCmd.first().command == "aa") {
         analLevel = 1;
-    } else if (options.analCmd.first().command == "aaa" ) {
+    } else if (options.analCmd.first().command == "aac") {
         analLevel = 2;
-    } else if (options.analCmd.first().command ==  "aaaa" ) {
+    } else if (options.analCmd.first().command == "aaa") {
         analLevel = 3;
-    } else {
+    } else if (options.analCmd.first().command == "aaaa") {
         analLevel = 4;
+    } else if (options.analCmd.first().command == "aaaaa") {
+        analLevel = 5;
+    } else {
+        analLevel = 6;
         AnalysisCommands item;
         QList<QString> commands = getAnalysisCommands(options);
-        foreach (item, analysisCommands){
+        foreach (item, analysisCommands) {
             qInfo() << item.commandDesc.command;
             item.checkbox->setChecked(commands.contains(item.commandDesc.command));
         }
@@ -189,20 +203,15 @@ void InitialOptionsDialog::loadOptions(const InitialOptions &options)
         ui->entry_loadOffset->setText(RAddressString(options.binLoadAddr));
     }
 
-	ui->writeCheckBox->setChecked(options.writeEnabled);
-
-
-    // TODO: all other options should also be applied to the ui
+    ui->writeCheckBox->setChecked(options.writeEnabled);
+    ui->varCheckBox->setChecked(Core()->getConfigb("anal.vars"));
 }
 
-
-void InitialOptionsDialog::setTooltipWithConfigHelp(QWidget *w, const char *config) {
-    w->setToolTip(QString("%1 (%2)")
-                  .arg(core->getConfigDescription(config))
-                  .arg(config));
+void InitialOptionsDialog::setTooltipWithConfigHelp(QWidget *w, const char *config)
+{
+    w->setToolTip(QStringLiteral("%1 (%2)").arg(core->getConfigDescription(config)).arg(config));
 }
 
-
 QString InitialOptionsDialog::getSelectedArch() const
 {
     QVariant archValue = ui->archComboBox->currentData();
@@ -249,10 +258,10 @@ QString InitialOptionsDialog::getSelectedOS() const
 QList<CommandDescription> InitialOptionsDialog::getSelectedAdvancedAnalCmds() const
 {
     QList<CommandDescription> advanced = QList<CommandDescription>();
-    if (ui->analSlider->value() == 3) {
+    if (ui->analSlider->value() == 7) {
         AnalysisCommands item;
-        foreach (item, analysisCommands){
-            if(item.checkbox->isChecked()) {
+        foreach (item, analysisCommands) {
+            if (item.checkbox->isChecked()) {
                 advanced << item.commandDesc;
             }
         }
@@ -260,7 +269,8 @@ QList<CommandDescription> InitialOptionsDialog::getSelectedAdvancedAnalCmds() co
     return advanced;
 }
 
-void InitialOptionsDialog::setupAndStartAnalysis(/*int level, QList<QString> advanced*/)
+void InitialOptionsDialog::setupAndStartAnalysis(
+    /*int level, QList<QString> advanced*/)
 {
     InitialOptions options;
 
@@ -275,11 +285,13 @@ void InitialOptionsDialog::setupAndStartAnalysis(/*int level, QList<QString> adv
         options.binLoadAddr = Core()->math(ui->entry_loadOffset->text());
     }
 
-    options.mapAddr = Core()->math(ui->entry_mapOffset->text());      // Where to map the file once loaded (-m)
+    // Where to map the file once loaded (-m)
+    options.mapAddr = Core()->math(ui->entry_mapOffset->text());
     options.arch = getSelectedArch();
     options.cpu = getSelectedCPU();
     options.bits = getSelectedBits();
     options.os = getSelectedOS();
+    options.analVars = ui->varCheckBox->isChecked();
     options.writeEnabled = ui->writeCheckBox->isChecked();
     options.loadBinInfo = !ui->binCheckBox->isChecked();
     options.loadBinCache = ui->binCacheCheckBox->isChecked();
@@ -296,21 +308,29 @@ void InitialOptionsDialog::setupAndStartAnalysis(/*int level, QList<QString> adv
         options.script = ui->scriptLineEdit->text();
     }
 
-
     options.endian = getSelectedEndianness();
 
     int level = ui->analSlider->value();
     switch (level) {
+    case 0:
+        options.analCmd = {};
+        break;
     case 1:
-        options.analCmd = { {"aa", "Auto analysis"} };
+        options.analCmd = {{"aa", "Auto analysis"}};
         break;
     case 2:
-        options.analCmd = { {"aaa", "Auto analysis (experimental)"} };
+        options.analCmd = {{"aac", "Analyse function calls"}};
         break;
     case 3:
-        options.analCmd = { {"aaaa", "Auto analysis (experimental)"} };
+        options.analCmd = {{"aaa", "Advanced Auto analysis"}};
         break;
     case 4:
+        options.analCmd = {{"aaaa", "Experimental analysis"}};
+        break;
+    case 5:
+        options.analCmd = {{"aaaaa", "Unstable analysis"}};
+        break;
+    case 6:
         options.analCmd = getSelectedAdvancedAnalCmds();
         break;
     default:
@@ -318,7 +338,6 @@ void InitialOptionsDialog::setupAndStartAnalysis(/*int level, QList<QString> adv
         break;
     }
 
-
     MainWindow *main = this->main;
 #if 0
     AnalTask *analTask = new AnalTask();
@@ -341,36 +360,45 @@ void InitialOptionsDialog::setupAndStartAnalysis(/*int level, QList<QString> adv
     Core()->getAsyncTaskManager()->start(analTaskPtr);
 #endif
 
-    done(0);
 #if MONOTHREAD
     int perms = R_PERM_RX;
     if (options.writeEnabled) {
         perms |= R_PERM_W;
-        emit Core()->ioModeChanged();
+        emit Core() -> ioModeChanged();
     }
 
     // Demangle (must be before file Core()->loadFile)
     Core()->setConfig("bin.demangle", options.demangle);
-
+    if (options.filename.endsWith("://") || options.filename == "") {
+        QMessageBox::warning(this, tr("Error"), tr("Please select a file"));
+        return;
+    }
+    if (options.filename.startsWith("malloc:")) {
+        options.loadBinInfo = false;
+    }
     // Do not reload the file if already loaded
     // QJsonArray openedFiles = Core()->getOpenedFiles();
     // if (true)  { // !openedFiles.size() && options.filename.length()) {
     if (options.filename.length()) {
-        bool fileLoaded = Core()->loadFile(options.filename,
-                                           options.binLoadAddr,
-                                           options.mapAddr,
-                                           perms,
-                                           options.useVA,
-                                           options.loadBinCache,
-                                           options.loadBinInfo,
-                                           options.forceBinPlugin);
+        bool fileLoaded = Core()->loadFile(
+            options.filename,
+            options.binLoadAddr,
+            options.mapAddr,
+            perms,
+            options.useVA,
+            options.loadBinCache,
+            options.loadBinInfo,
+            options.forceBinPlugin);
         if (!fileLoaded) {
-//            emit openFileFailed();
+            //            emit openFileFailed();
+            QMessageBox::warning(this, tr("Error"), tr("Cannot open"));
             return;
         }
     }
+    done(0);
 
-    // r_core_bin_load might change asm.bits, so let's set that after the bin is loaded
+    // r_core_bin_load might change asm.bits, so let's set that after the bin is
+    // loaded
     Core()->setCPU(options.arch, options.cpu, options.bits);
 
     if (!options.os.isNull()) {
@@ -378,40 +406,37 @@ void InitialOptionsDialog::setupAndStartAnalysis(/*int level, QList<QString> adv
     }
 
     if (!options.pdbFile.isNull()) {
-   //     log(tr("Loading PDB file..."));
+        //     log(tr("Loading PDB file..."));
         Core()->loadPDB(options.pdbFile);
     }
 
     if (!options.shellcode.isNull() && options.shellcode.size() / 2 > 0) {
-  //      log(tr("Loading shellcode..."));
+        //      log(tr("Loading shellcode..."));
         Core()->cmdRaw("wx " + options.shellcode);
     }
 
     if (options.endian != InitialOptions::Endianness::Auto) {
         Core()->setEndianness(options.endian == InitialOptions::Endianness::Big);
     }
+    ui->varCheckBox->setChecked(Core()->getConfigb("anal.vars"));
 
     Core()->cmdRaw("fs *");
 
     if (!options.script.isNull()) {
- //       log(tr("Executing script..."));
+        //       log(tr("Executing script..."));
         Core()->loadScript(options.script);
     }
 
+    const bool boi = Core()->getConfigb("esil.breakoninvalid");
+    Config()->setConfig("esil.breakoninvalid", false);
     if (!options.analCmd.empty()) {
- //       log(tr("Executing analysis..."));
         for (const CommandDescription &cmd : options.analCmd) {
-            // log(cmd.description);
-          //  log(cmd.command + " : " + cmd.description);
-            // use cmd instead of cmdRaw because commands can be unexpected
             Core()->cmd(cmd.command);
         }
-        // log(tr("Analysis complete!"));
-    } else {
-//        log(tr("Skipping Analysis."));
+        Core()->setConfig("esil.breakoninvalid", boi);
     }
 #endif
-        main->finalizeOpen();
+    main->finalizeOpen();
 }
 
 void InitialOptionsDialog::on_okButton_clicked()
@@ -427,18 +452,22 @@ void InitialOptionsDialog::closeEvent(QCloseEvent *event)
 
 QString InitialOptionsDialog::analysisDescription(int level)
 {
-    //TODO: replace this with meaningful descriptions
+    // TODO: replace this with meaningful descriptions
     switch (level) {
     case 0:
         return tr("No analysis");
     case 1:
         return tr("Auto-Analysis (aa)");
     case 2:
-        return tr("Auto-Analysis (aaa)");
+        return tr("Call-Analysis (aac)");
     case 3:
-        return tr("Auto-Analysis Experimental (aaaa)");
+        return tr("Advanced Analysis (aaa)");
     case 4:
-        return tr("Advanced");
+        return tr("Experimental Analysis (aaaa)");
+    case 5:
+        return tr("Cutting Edge Analysis (aaaaa)");
+    case 6:
+        return tr("Custom Analysis Steps");
     default:
         return tr("Unknown");
     }
@@ -446,14 +475,15 @@ QString InitialOptionsDialog::analysisDescription(int level)
 
 void InitialOptionsDialog::on_analSlider_valueChanged(int value)
 {
-    ui->analDescription->setText(tr("Level") + QString(": %1").arg(analysisDescription(value)));
+    ui->analDescription->setText(
+        tr("Level") + QStringLiteral(": %1").arg(analysisDescription(value)));
     if (value == 0) {
         ui->analCheckBox->setChecked(false);
         ui->analCheckBox->setText(tr("Analysis: Disabled"));
     } else {
         ui->analCheckBox->setChecked(true);
-        ui->analCheckBox->setText(tr("Analysis: Enabled"));
-        if (value == 4) {
+        ui->analCheckBox->setText(tr("Run Analysis"));
+        if (value == 6) {
             ui->analoptionsFrame->setVisible(true);
             ui->advancedAnlysisLine->setVisible(true);
         } else {
@@ -496,7 +526,7 @@ void InitialOptionsDialog::on_pdbSelectButton_clicked()
 {
     QFileDialog dialog(this);
     dialog.setWindowTitle(tr("Select PDB file"));
-    dialog.setNameFilters({ tr("PDB file (*.pdb)"), tr("All files (*)") });
+    dialog.setNameFilters({tr("PDB file (*.pdb)"), tr("All files (*)")});
 
     if (!dialog.exec()) {
         return;
@@ -509,7 +539,6 @@ void InitialOptionsDialog::on_pdbSelectButton_clicked()
     }
 }
 
-
 void InitialOptionsDialog::updateScriptLayout()
 {
     ui->scriptWidget->setEnabled(ui->scriptCheckBox->isChecked());
@@ -519,7 +548,7 @@ void InitialOptionsDialog::on_scriptSelectButton_clicked()
 {
     QFileDialog dialog(this);
     dialog.setWindowTitle(tr("Select radare2 script file"));
-    dialog.setNameFilters({ tr("Script file (*.r2)"), tr("All files (*)") });
+    dialog.setNameFilters({tr("Script file (*.r2)"), tr("All files (*)")});
 
     if (!dialog.exec()) {
         return;
@@ -532,7 +561,6 @@ void InitialOptionsDialog::on_scriptSelectButton_clicked()
     }
 }
 
-
 void InitialOptionsDialog::reject()
 {
     done(0);
